///////////////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2006-2012 Esper Team. All rights reserved.                           /
// http://esper.codehaus.org                                                          /
// ---------------------------------------------------------------------------------- /
// The software in this package is published under the terms of the GPL license       /
// a copy of which has been included with this distribution in the license.txt file.  /
///////////////////////////////////////////////////////////////////////////////////////

using System.Data;
using System.Data.Common;

using com.espertech.esper.client;
using com.espertech.esper.compat;
using com.espertech.esper.support.epl;

using NUnit.Framework;

namespace com.espertech.esper.epl.db
{
    [TestFixture]
    public class TestDatabaseDMConnFactory 
    {
        private DatabaseDriverConnFactory _databaseDmConnFactoryOne;
        private DatabaseDriverConnFactory _databaseDmConnFactoryTwo;
        private DatabaseDriverConnFactory _databaseDmConnFactoryThree;
    
        [SetUp]
        public void SetUp()
        {
            Properties properties = new Properties();

            // driver-manager config 1
            ConfigurationDBRef config = new ConfigurationDBRef();
            // Server=myServerAddress;Database=myDataBase;Trusted_Connection=True
            properties["Server"] = System.Net.Dns.GetHostName();
            properties["Database"] = "tempdb";
            properties["Trusted_Connection"] = "True";
            config.SetDatabaseDriver(SupportDatabaseService.DbDriverFactory1, properties);
            config.ConnectionCatalog = "test";
            config.ConnectionAutoCommit = false; // not supported yet
            config.ConnectionTransactionIsolation = IsolationLevel.Unspecified;
            _databaseDmConnFactoryOne = new DatabaseDriverConnFactory((DbDriverFactoryConnection)config.ConnectionFactoryDesc, config.ConnectionSettings);

            // driver-manager config 2
            config = new ConfigurationDBRef();
            config.SetDatabaseDriver(SupportDatabaseService.DbDriverFactory1);
            _databaseDmConnFactoryTwo = new DatabaseDriverConnFactory((DbDriverFactoryConnection)config.ConnectionFactoryDesc, config.ConnectionSettings);

            // driver-manager config 3
            config = new ConfigurationDBRef();
            config.SetDatabaseDriver(SupportDatabaseService.DbDriverFactory2);
            _databaseDmConnFactoryThree = new DatabaseDriverConnFactory((DbDriverFactoryConnection)config.ConnectionFactoryDesc, config.ConnectionSettings);
        }
    
        [Test]
        public void TestGetConnection()
        {
            DbConnection connection = _databaseDmConnFactoryOne.Driver.CreateConnection();
            TryAndCloseConnection(connection);

            connection = _databaseDmConnFactoryTwo.Driver.CreateConnection();
            TryAndCloseConnection(connection);

            connection = _databaseDmConnFactoryThree.Driver.CreateConnection();
            TryAndCloseConnection(connection);
        }

        private static void TryAndCloseConnection(DbConnection connection)
        {
            DbCommand stmt;

            stmt = connection.CreateCommand();
            stmt.CommandText = "select 1 from dual";
            stmt.CommandType = CommandType.Text;

            using (DbDataReader result = stmt.ExecuteReader())
            {
                Assert.IsTrue(result.Read());
                Assert.AreEqual(1, result.GetInt32(0));
                result.Close();
            }

            stmt.Dispose();

            connection.Close();
            connection.Dispose();
        }
    }
}
