///////////////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2006-2012 Esper Team. All rights reserved.                           /
// http://esper.codehaus.org                                                          /
// ---------------------------------------------------------------------------------- /
// The software in this package is published under the terms of the GPL license       /
// a copy of which has been included with this distribution in the license.txt file.  /
///////////////////////////////////////////////////////////////////////////////////////

using System;

using com.espertech.esper.epl.expression.core;
using com.espertech.esper.epl.expression.funcs;
using com.espertech.esper.epl.expression.ops;
using com.espertech.esper.support.epl;

using NUnit.Framework;


namespace com.espertech.esper.epl.expression
{
    [TestFixture]
    public class TestExprCoalesceNode 
    {
        private ExprCoalesceNode[] coalesceNodes;
    
        [SetUp]
        public void SetUp()
        {
            coalesceNodes = new ExprCoalesceNode[5];
    
            coalesceNodes[0] = new ExprCoalesceNode();
            coalesceNodes[0].AddChildNode(new SupportExprNode(null, typeof(long)));
            coalesceNodes[0].AddChildNode(new SupportExprNode(null, typeof(int)));
            coalesceNodes[0].AddChildNode(new SupportExprNode(4, typeof(byte)));
    
            coalesceNodes[1] = new ExprCoalesceNode();
            coalesceNodes[1].AddChildNode(new SupportExprNode(null, typeof(string)));
            coalesceNodes[1].AddChildNode(new SupportExprNode("a", typeof(string)));
    
            coalesceNodes[2] = new ExprCoalesceNode();
            coalesceNodes[2].AddChildNode(new SupportExprNode(null, typeof(Boolean)));
            coalesceNodes[2].AddChildNode(new SupportExprNode(true, typeof(bool)));
    
            coalesceNodes[3] = new ExprCoalesceNode();
            coalesceNodes[3].AddChildNode(new SupportExprNode(null, typeof(char)));
            coalesceNodes[3].AddChildNode(new SupportExprNode(null, typeof(char)));
            coalesceNodes[3].AddChildNode(new SupportExprNode(null, typeof(char)));
            coalesceNodes[3].AddChildNode(new SupportExprNode('b', typeof(char)));
    
            coalesceNodes[4] = new ExprCoalesceNode();
            coalesceNodes[4].AddChildNode(new SupportExprNode(5, typeof(float)));
            coalesceNodes[4].AddChildNode(new SupportExprNode(null, typeof(double?)));
        }
    
        [Test]
        public void TestGetType()
        {
            for (int i = 0; i < coalesceNodes.Length; i++)
            {
                coalesceNodes[i].Validate(ExprValidationContextFactory.MakeEmpty());
            }

            Assert.AreEqual(typeof(long?), coalesceNodes[0].ReturnType);
            Assert.AreEqual(typeof(string), coalesceNodes[1].ReturnType);
            Assert.AreEqual(typeof(bool?), coalesceNodes[2].ReturnType);
            Assert.AreEqual(typeof(char?), coalesceNodes[3].ReturnType);
            Assert.AreEqual(typeof(double?), coalesceNodes[4].ReturnType);
        }
    
        [Test]
        public void TestValidate()
        {
            ExprCoalesceNode coalesceNode = new ExprCoalesceNode();
            coalesceNode.AddChildNode(new SupportExprNode(1));
    
            // Test too few nodes under this node
            try
            {
                coalesceNode.Validate(ExprValidationContextFactory.MakeEmpty());
                Assert.Fail();
            }
            catch (ExprValidationException ex)
            {
                // Expected
            }
    
            // Test node result type not fitting
            coalesceNode.AddChildNode(new SupportExprNode("s"));
            try
            {
                coalesceNode.Validate(ExprValidationContextFactory.MakeEmpty());
                Assert.Fail();
            }
            catch (ExprValidationException ex)
            {
                // Expected
            }
        }
    
        [Test]
        public void TestEvaluate()
        {
            for (int i = 0; i < coalesceNodes.Length; i++)
            {
                coalesceNodes[i].Validate(ExprValidationContextFactory.MakeEmpty());
            }
    
            Assert.AreEqual(4L, coalesceNodes[0].Evaluate(new EvaluateParams(null, false, null)));
            Assert.AreEqual("a", coalesceNodes[1].Evaluate(new EvaluateParams(null, false, null)));
            Assert.AreEqual(true, coalesceNodes[2].Evaluate(new EvaluateParams(null, false, null)));
            Assert.AreEqual('b', coalesceNodes[3].Evaluate(new EvaluateParams(null, false, null)));
            Assert.AreEqual(5D, coalesceNodes[4].Evaluate(new EvaluateParams(null, false, null)));
        }
    
        [Test]
        public void TestEquals()
        {
            Assert.IsFalse(coalesceNodes[0].EqualsNode(new ExprEqualsNodeImpl(true, false)));
            Assert.IsTrue(coalesceNodes[0].EqualsNode(coalesceNodes[1]));
        }
    
        [Test]
        public void TestToExpressionString()
        {
            coalesceNodes[0].Validate(ExprValidationContextFactory.MakeEmpty());
            Assert.AreEqual("coalesce(null,null,4)", coalesceNodes[0].ToExpressionStringMinPrecedence());
        }
    }
}
