///////////////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2006-2014 Esper Team. All rights reserved.                           /
// http://esper.codehaus.org                                                          /
// ---------------------------------------------------------------------------------- /
// The software in this package is published under the terms of the GPL license       /
// a copy of which has been included with this distribution in the license.txt file.  /
///////////////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Threading;

using com.espertech.esper.client;
using com.espertech.esper.compat;
using com.espertech.esper.support.client;

using NUnit.Framework;

namespace com.espertech.esper.regression.context
{
    [TestFixture]
    public class TestContextMTTerminated
    {
        [Test]
        public void TestMTTerminateFault()
        {
            var config = SupportConfigFactory.GetConfiguration();
            config.EngineDefaults.ThreadingConfig.IsInternalTimerEnabled = true;
            config.AddEventType(typeof(StartContextEvent));
            config.AddEventType(typeof(PayloadEvent));
            var epService = EPServiceProviderManager.GetDefaultProvider(config);
            epService.Initialize();
    
            var eplStatement = "create context StartThenTwoSeconds start StartContextEvent end after 2 seconds";
            epService.EPAdministrator.CreateEPL(eplStatement);
    
            var aggStatement = "@Name('select') context StartThenTwoSeconds " +
                    "select account, count(*) as totalCount " +
                    "from PayloadEvent " +
                    "group by account " +
                    "output snapshot when terminated";
            var epAggStatement = epService.EPAdministrator.CreateEPL(aggStatement);
            epAggStatement.Events += (sender, args) => {
                // no action, still listening to make sure select-clause evaluates
            };
    
            // start context
            epService.EPRuntime.SendEvent(new StartContextEvent());
    
            // start threads
            var threads = new List<Thread>();
            var runnables = new List<MyRunnable>();
            for (var i = 0; i < 8; i++) {
                var myRunnable = new MyRunnable(epService);
                runnables.Add(myRunnable);
                var thread = new Thread(myRunnable.Run);
                thread.Name = "Thread" + i;
                thread.Start();
                threads.Add(thread);
            }
    
            // join
            threads.ForEach(t => t.Join());
    
            // assert
            runnables.ForEach(r => Assert.IsNull(r.Exception));
        }
    
        public class StartContextEvent {}
        
        public class PayloadEvent
        {
            public PayloadEvent(string account)
            {
                Account = account;
            }

            public string Account { get; private set; }
        }
    
        public class MyRunnable
        {
            private readonly EPServiceProvider _engine;

            public Exception Exception { get; private set; }

            public MyRunnable(EPServiceProvider engine)
            {
                _engine = engine;
            }
    
            public void Run()
            {
                try
                {
                    for (var i = 0; i < 2000000; i++)
                    {
                        var payloadEvent = new PayloadEvent("A1");
                        _engine.EPRuntime.SendEvent(payloadEvent);
                    }
                }
                catch (Exception ex)
                {
                    Exception = ex;
                }
            }
        }
    }
}
