///////////////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2006-2012 Esper Team. All rights reserved.                           /
// http://esper.codehaus.org                                                          /
// ---------------------------------------------------------------------------------- /
// The software in this package is published under the terms of the GPL license       /
// a copy of which has been included with this distribution in the license.txt file.  /
///////////////////////////////////////////////////////////////////////////////////////

using System;
using System.IO;
using System.Xml;
using com.espertech.esper.client;
using com.espertech.esper.client.scopetest;
using com.espertech.esper.client.util;
using com.espertech.esper.compat;
using com.espertech.esper.regression.events;
using com.espertech.esper.support.client;

using NUnit.Framework;

namespace com.espertech.esper.regression.view
{
    [TestFixture]
    public class TestFilterPropertyExample  {
        private EPServiceProvider epService;
        private XmlDocument eventDocOne;
        private XmlDocument eventDocTwo;
    
        [SetUp]
        public void SetUp() {
            epService = EPServiceProviderManager.GetDefaultProvider(SupportConfigFactory.GetConfiguration());
            epService.Initialize();
    
            ConfigurationEventTypeXMLDOM config = new ConfigurationEventTypeXMLDOM();
            String schemaUri = ResourceManager.ResolveResourceURL("regression/mediaOrderSchema.xsd").ToString();
            config.SchemaResource = schemaUri;
            config.RootElementName = "mediaorder";
    
            epService.EPAdministrator.Configuration.AddEventType("MediaOrder", config);
            epService.EPAdministrator.Configuration.AddEventType("Cancel", config);
    
            Stream xmlStreamOne = ResourceManager.GetResourceAsStream("regression/mediaOrderOne.xml");
            eventDocOne = SupportXML.GetDocument(xmlStreamOne);

            Stream xmlStreamTwo = ResourceManager.GetResourceAsStream("regression/mediaOrderTwo.xml");
            eventDocTwo = SupportXML.GetDocument(xmlStreamTwo);
        }
    
        [TearDown]
        public void TearDown() {
            eventDocOne = null;
            eventDocTwo = null;
        }
    
        [Test]
        public void TestExample() {
            String stmtTextOne = "select orderId, items.item[0].itemId from MediaOrder";
            EPStatement stmtOne = epService.EPAdministrator.CreateEPL(stmtTextOne);
            SupportUpdateListener listenerOne = new SupportUpdateListener();
            stmtOne.Events += listenerOne.Update;
    
            String stmtTextTwo = "select * from MediaOrder[books.book]";
            EPStatement stmtTwo = epService.EPAdministrator.CreateEPL(stmtTextTwo);
            SupportUpdateListener listenerTwo = new SupportUpdateListener();
            stmtTwo.Events += listenerTwo.Update;
    
            String stmtTextThree = "select * from MediaOrder(orderId='PO200901')[books.book]";
            EPStatement stmtThree = epService.EPAdministrator.CreateEPL(stmtTextThree);
            SupportUpdateListener listenerThree = new SupportUpdateListener();
            stmtThree.Events += listenerThree.Update;
    
            String stmtTextFour = "select count(*) from MediaOrder[books.book].std:unique(bookId)";
            EPStatement stmtFour = epService.EPAdministrator.CreateEPL(stmtTextFour);
            SupportUpdateListener listenerFour = new SupportUpdateListener();
            stmtFour.Events += listenerFour.Update;
    
            String stmtTextFive = "select * from MediaOrder[books.book][review]";
            EPStatement stmtFive = epService.EPAdministrator.CreateEPL(stmtTextFive);
            SupportUpdateListener listenerFive = new SupportUpdateListener();
            stmtFive.Events += listenerFive.Update;
    
            String stmtTextSix = "select * from pattern [c=Cancel -> o=MediaOrder(orderId = c.orderId)[books.book]]";
            EPStatement stmtSix = epService.EPAdministrator.CreateEPL(stmtTextSix);
            SupportUpdateListener listenerSix = new SupportUpdateListener();
            stmtSix.Events += listenerSix.Update;
    
            String stmtTextSeven = "select * from MediaOrder[select orderId, bookId from books.book][select * from review]";
            EPStatement stmtSeven = epService.EPAdministrator.CreateEPL(stmtTextSeven);
            SupportUpdateListener listenerSeven = new SupportUpdateListener();
            stmtSeven.Events += listenerSeven.Update;
    
            String stmtTextEight = "select * from MediaOrder[select * from books.book][select reviewId, comment from review]";
            EPStatement stmtEight = epService.EPAdministrator.CreateEPL(stmtTextEight);
            SupportUpdateListener listenerEight = new SupportUpdateListener();
            stmtEight.Events += listenerEight.Update;
    
            String stmtTextNine = "select * from MediaOrder[books.book as book][select book.*, reviewId, comment from review]";
            EPStatement stmtNine = epService.EPAdministrator.CreateEPL(stmtTextNine);
            SupportUpdateListener listenerNine = new SupportUpdateListener();
            stmtNine.Events += listenerNine.Update;
    
            String stmtTextTen = "select * from MediaOrder[books.book as book][select mediaOrder.*, bookId, reviewId from review] as mediaOrder";
            EPStatement stmtTen = epService.EPAdministrator.CreateEPL(stmtTextTen);
            SupportUpdateListener listenerTen = new SupportUpdateListener();
            stmtTen.Events += listenerTen.Update;
    
            String stmtTextEleven_0 = "insert into ReviewStream select * from MediaOrder[books.book as book]\n" +
                    "    [select mediaOrder.* as mediaOrder, book.* as book, review.* as review from review as review] as mediaOrder";
            epService.EPAdministrator.CreateEPL(stmtTextEleven_0);
            String stmtTextEleven_1 = "select mediaOrder.orderId, book.bookId, review.reviewId from ReviewStream";
            EPStatement stmtEleven_1 = epService.EPAdministrator.CreateEPL(stmtTextEleven_1);
            SupportUpdateListener listenerEleven = new SupportUpdateListener();
            stmtEleven_1.Events += listenerEleven.Update;
    
            String stmtTextTwelve = "select * from MediaOrder[books.book where author = 'Orson Scott Card'][review]";
            EPStatement stmtTwelve = epService.EPAdministrator.CreateEPL(stmtTextTwelve);
            SupportUpdateListener listenerTwelve = new SupportUpdateListener();
            stmtTwelve.Events += listenerTwelve.Update;
    
            epService.EPRuntime.SendEvent(eventDocOne);
    
            EPAssertionUtil.AssertProps(listenerOne.AssertOneGetNewAndReset(), "orderId,items.item[0].itemId".Split(','), new Object[]{"PO200901", "100001"});
            EPAssertionUtil.AssertPropsPerRow(listenerTwo.LastNewData, "bookId".Split(','), new Object[][]{new Object[] {"B001"}, new Object[] {"B002"}});
            EPAssertionUtil.AssertPropsPerRow(listenerThree.LastNewData, "bookId".Split(','), new Object[][]{new Object[] {"B001"}, new Object[] {"B002"}});
            EPAssertionUtil.AssertPropsPerRow(listenerFour.LastNewData, "count(*)".Split(','), new Object[][]{new Object[] {2L}});
            EPAssertionUtil.AssertPropsPerRow(listenerFive.LastNewData, "reviewId".Split(','), new Object[][]{new Object[] {"1"}});
            Assert.IsFalse(listenerSix.IsInvoked);
            EPAssertionUtil.AssertPropsPerRow(listenerSeven.LastNewData, "orderId,bookId,reviewId".Split(','), new Object[][]{new Object[] {"PO200901", "B001", "1"}});
            EPAssertionUtil.AssertPropsPerRow(listenerEight.LastNewData, "reviewId,bookId".Split(','), new Object[][]{new Object[] {"1", "B001"}});
            EPAssertionUtil.AssertPropsPerRow(listenerNine.LastNewData, "reviewId,bookId".Split(','), new Object[][]{new Object[] {"1", "B001"}});
            EPAssertionUtil.AssertPropsPerRow(listenerTen.LastNewData, "reviewId,bookId".Split(','), new Object[][]{new Object[] {"1", "B001"}});
            EPAssertionUtil.AssertPropsPerRow(listenerEleven.LastNewData, "mediaOrder.orderId,book.bookId,review.reviewId".Split(','), new Object[][]{new Object[] {"PO200901", "B001", "1"}});
            EPAssertionUtil.AssertPropsPerRow(listenerTwelve.LastNewData, "reviewId".Split(','), new Object[][]{new Object[] {"1"}});
        }
    
        [Test]
        public void TestJoinSelfJoin() {
            String stmtText = "select book.bookId,item.itemId from MediaOrder[books.book] as book, MediaOrder[items.item] as item where productId = bookId order by bookId, item.itemId asc";
            EPStatement stmt = epService.EPAdministrator.CreateEPL(stmtText);
            SupportUpdateListener listener = new SupportUpdateListener();
            stmt.Events += listener.Update;
    
            String[] fields = "book.bookId,item.itemId".Split(',');
            epService.EPRuntime.SendEvent(eventDocOne);
            PrintRows(listener.LastNewData);
            EPAssertionUtil.AssertPropsPerRow(listener.GetAndResetLastNewData(), fields, new Object[][]{new Object[] {"B001", "100001"}});
    
            epService.EPRuntime.SendEvent(eventDocOne);
            EPAssertionUtil.AssertPropsPerRow(listener.GetAndResetLastNewData(), fields, new Object[][]{new Object[] {"B001", "100001"}});
    
            epService.EPRuntime.SendEvent(eventDocTwo);
            EPAssertionUtil.AssertPropsPerRow(listener.GetAndResetLastNewData(), fields, new Object[][]{new Object[] {"B005", "200002"}, new Object[] {"B005", "200004"}, new Object[] {"B006", "200001"}});
    
            // count
            stmt.Dispose();
            fields = "count(*)".Split(',');
            stmtText = "select count(*) from MediaOrder[books.book] as book, MediaOrder[items.item] as item where productId = bookId order by bookId asc";
            stmt = epService.EPAdministrator.CreateEPL(stmtText);
            stmt.Events += listener.Update;
    
            epService.EPRuntime.SendEvent(eventDocTwo);
            EPAssertionUtil.AssertPropsPerRow(listener.GetAndResetLastNewData(), fields, new Object[][]{new Object[] {3L}});
    
            epService.EPRuntime.SendEvent(eventDocOne);
            EPAssertionUtil.AssertPropsPerRow(listener.GetAndResetLastNewData(), fields, new Object[][]{new Object[] {4L}});
    
            // unidirectional count
            stmt.Dispose();
            stmtText = "select count(*) from MediaOrder[books.book] as book unidirectional, MediaOrder[items.item] as item where productId = bookId order by bookId asc";
            stmt = epService.EPAdministrator.CreateEPL(stmtText);
            stmt.Events += listener.Update;
    
            epService.EPRuntime.SendEvent(eventDocTwo);
            EPAssertionUtil.AssertPropsPerRow(listener.GetAndResetLastNewData(), fields, new Object[][]{new Object[] {3L}});
    
            epService.EPRuntime.SendEvent(eventDocOne);
            EPAssertionUtil.AssertPropsPerRow(listener.GetAndResetLastNewData(), fields, new Object[][]{new Object[] {1L}});
        }
    
        [Test]
        public void TestJoinSelfLeftOuterJoin() {
            String stmtText = "select book.bookId,item.itemId from MediaOrder[books.book] as book left outer join MediaOrder[items.item] as item on productId = bookId order by bookId, item.itemId asc";
            EPStatement stmt = epService.EPAdministrator.CreateEPL(stmtText);
            SupportUpdateListener listener = new SupportUpdateListener();
            stmt.Events += listener.Update;
    
            String[] fields = "book.bookId,item.itemId".Split(',');
            epService.EPRuntime.SendEvent(eventDocTwo);
            EPAssertionUtil.AssertPropsPerRow(listener.GetAndResetLastNewData(), fields, new Object[][]{new Object[] {"B005", "200002"}, new Object[] {"B005", "200004"}, new Object[] {"B006", "200001"}, new Object[] {"B008", null}});
    
            epService.EPRuntime.SendEvent(eventDocOne);
            PrintRows(listener.LastNewData);
            EPAssertionUtil.AssertPropsPerRow(listener.GetAndResetLastNewData(), fields, new Object[][]{new Object[] {"B001", "100001"}, new Object[] {"B002", null}});
    
            // count
            stmt.Dispose();
            fields = "count(*)".Split(',');
            stmtText = "select count(*) from MediaOrder[books.book] as book left outer join MediaOrder[items.item] as item on productId = bookId";
            stmt = epService.EPAdministrator.CreateEPL(stmtText);
            stmt.Events += listener.Update;
    
            epService.EPRuntime.SendEvent(eventDocTwo);
            EPAssertionUtil.AssertPropsPerRow(listener.GetAndResetLastNewData(), fields, new Object[][]{new Object[] {4L}});
    
            epService.EPRuntime.SendEvent(eventDocOne);
            EPAssertionUtil.AssertPropsPerRow(listener.GetAndResetLastNewData(), fields, new Object[][]{new Object[] {6L}});
    
            // unidirectional count
            stmt.Dispose();
            stmtText = "select count(*) from MediaOrder[books.book] as book unidirectional left outer join MediaOrder[items.item] as item on productId = bookId";
            stmt = epService.EPAdministrator.CreateEPL(stmtText);
            stmt.Events += listener.Update;
    
            epService.EPRuntime.SendEvent(eventDocTwo);
            EPAssertionUtil.AssertPropsPerRow(listener.GetAndResetLastNewData(), fields, new Object[][]{new Object[] {4L}});
    
            epService.EPRuntime.SendEvent(eventDocOne);
            EPAssertionUtil.AssertPropsPerRow(listener.GetAndResetLastNewData(), fields, new Object[][]{new Object[] {2L}});
        }
    
        [Test]
        public void TestJoinSelfFullOuterJoin() {
            String stmtText = "select orderId, book.bookId,item.itemId from MediaOrder[books.book] as book full outer join MediaOrder[select orderId, * from items.item] as item on productId = bookId order by bookId, item.itemId asc";
            EPStatement stmt = epService.EPAdministrator.CreateEPL(stmtText);
            SupportUpdateListener listener = new SupportUpdateListener();
            stmt.Events += listener.Update;
    
            String[] fields = "book.bookId,item.itemId".Split(',');
            epService.EPRuntime.SendEvent(eventDocTwo);
            EPAssertionUtil.AssertPropsPerRow(listener.GetAndResetLastNewData(), fields, new Object[][]{new Object[] {null, "200003"}, new Object[] {"B005", "200002"}, new Object[] {"B005", "200004"}, new Object[] {"B006", "200001"}, new Object[] {"B008", null}});
    
            epService.EPRuntime.SendEvent(eventDocOne);
            PrintRows(listener.LastNewData);
            EPAssertionUtil.AssertPropsPerRow(listener.GetAndResetLastNewData(), fields, new Object[][]{new Object[] {"B001", "100001"}, new Object[] {"B002", null}});
    
            // count
            stmt.Dispose();
            fields = "count(*)".Split(',');
            stmtText = "select count(*) from MediaOrder[books.book] as book full outer join MediaOrder[items.item] as item on productId = bookId";
            stmt = epService.EPAdministrator.CreateEPL(stmtText);
            stmt.Events += listener.Update;
    
            epService.EPRuntime.SendEvent(eventDocTwo);
            EPAssertionUtil.AssertPropsPerRow(listener.GetAndResetLastNewData(), fields, new Object[][]{new Object[] {5L}});
    
            epService.EPRuntime.SendEvent(eventDocOne);
            EPAssertionUtil.AssertPropsPerRow(listener.GetAndResetLastNewData(), fields, new Object[][]{new Object[] {7L}});
    
            // unidirectional count
            stmt.Dispose();
            stmtText = "select count(*) from MediaOrder[books.book] as book unidirectional full outer join MediaOrder[items.item] as item on productId = bookId";
            stmt = epService.EPAdministrator.CreateEPL(stmtText);
            stmt.Events += listener.Update;
    
            epService.EPRuntime.SendEvent(eventDocTwo);
            EPAssertionUtil.AssertPropsPerRow(listener.GetAndResetLastNewData(), fields, new Object[][]{new Object[] {4L}});
    
            epService.EPRuntime.SendEvent(eventDocOne);
            EPAssertionUtil.AssertPropsPerRow(listener.GetAndResetLastNewData(), fields, new Object[][]{new Object[] {2L}});
        }
    
        private void PrintRows(EventBean[] rows) {
            JSONEventRenderer renderer = epService.EPRuntime.EventRenderer.GetJSONRenderer(rows[0].EventType);
            for (int i = 0; i < rows.Length; i++) {
                // Console.WriteLine(renderer.Render("event#" + i, rows[i]));
                renderer.Render("event#" + i, rows[i]);
            }
        }
    
        [Test]
        public void TestSolutionPattern() {
            epService.EPAdministrator.Configuration.AddEventType("ResponseEvent", typeof(ResponseEvent));
    
            String[] fields = "category,subEventType,avgTime".Split(',');
            String stmtText = "select category, subEventType, avg(responseTimeMillis) as avgTime from ResponseEvent[select category, * from subEvents].win:time(1 min) group by category, subEventType order by category, subEventType";
            EPStatement stmt = epService.EPAdministrator.CreateEPL(stmtText);
            SupportUpdateListener listener = new SupportUpdateListener();
            stmt.Events += listener.Update;
    
            epService.EPRuntime.SendEvent(new ResponseEvent("svcOne", new SubEvent[]{new SubEvent(1000, "typeA"), new SubEvent(800, "typeB")}));
            EPAssertionUtil.AssertPropsPerRow(listener.GetAndResetLastNewData(), fields, new Object[][]{new Object[] {"svcOne", "typeA", 1000.0}, new Object[] {"svcOne", "typeB", 800.0}});
    
            epService.EPRuntime.SendEvent(new ResponseEvent("svcOne", new SubEvent[]{new SubEvent(400, "typeB"), new SubEvent(500, "typeA")}));
            EPAssertionUtil.AssertPropsPerRow(listener.GetAndResetLastNewData(), fields, new Object[][]{new Object[] {"svcOne", "typeA", 750.0}, new Object[] {"svcOne", "typeB", 600.0}});
        }
    
        public class ResponseEvent {
            private String category;
            private SubEvent[] subEvents;
    
            public ResponseEvent(String category, SubEvent[] subEvents) {
                this.category = category;
                this.subEvents = subEvents;
            }
    
            public String GetCategory() {
                return category;
            }
    
            public SubEvent[] GetSubEvents() {
                return subEvents;
            }
        }
    
        public class SubEvent {
            private long responseTimeMillis;
            private String subEventType;
    
            public SubEvent(long responseTimeMillis, String subEventType) {
                this.responseTimeMillis = responseTimeMillis;
                this.subEventType = subEventType;
            }
    
            public long GetResponseTimeMillis() {
                return responseTimeMillis;
            }
    
            public String GetSubEventType() {
                return subEventType;
            }
        }
    }
}
