///////////////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2006-2012 Esper Team. All rights reserved.                           /
// http://esper.codehaus.org                                                          /
// ---------------------------------------------------------------------------------- /
// The software in this package is published under the terms of the GPL license       /
// a copy of which has been included with this distribution in the license.txt file.  /
///////////////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;

using com.espertech.esper.client;
using com.espertech.esper.client.scopetest;
using com.espertech.esper.client.soda;
using com.espertech.esper.metrics.instrumentation;
using com.espertech.esper.support.bean;
using com.espertech.esper.support.client;
using com.espertech.esper.support.util;

using NUnit.Framework;

namespace com.espertech.esper.regression.view
{
    using DataMap = IDictionary<string, object>;

    [TestFixture]
    public class TestNewExpr 
    {
        private EPServiceProvider _epService;
        private SupportUpdateListener _listener;
    
        [SetUp]
        public void SetUp()
        {
            _listener = new SupportUpdateListener();
            _epService = EPServiceProviderManager.GetDefaultProvider(SupportConfigFactory.GetConfiguration());
            _epService.Initialize();
            if (InstrumentationHelper.ENABLED) { InstrumentationHelper.StartTest(_epService, GetType(), GetType().FullName); }
            _epService.EPAdministrator.Configuration.AddEventType("SupportBean", typeof(SupportBean));
        }

        [TearDown]
        public void TearDown()
        {
            if (InstrumentationHelper.ENABLED) { InstrumentationHelper.EndTest(); }
            _listener = null;
        }
    
        [Test]
        public void TestNewAlone()
        {
            String epl = "select new { TheString = 'x' || TheString || 'x', IntPrimitive = IntPrimitive + 2} as val0 from SupportBean as sb";
    
            EPStatement stmt = _epService.EPAdministrator.CreateEPL(epl);
            stmt.Events += _listener.Update;
    
            Assert.AreEqual(typeof(DataMap), stmt.EventType.GetPropertyType("val0"));
            FragmentEventType fragType = stmt.EventType.GetFragmentType("val0");
            Assert.IsFalse(fragType.IsIndexed);
            Assert.IsFalse(fragType.IsNative);
            Assert.AreEqual(typeof(string), fragType.FragmentType.GetPropertyType("TheString"));
            Assert.AreEqual(typeof(int?), fragType.FragmentType.GetPropertyType("IntPrimitive"));

            String[] fieldsInner = "TheString,IntPrimitive".Split(',');
            _epService.EPRuntime.SendEvent(new SupportBean("E1", -5));
            ArrayAssertionUtil.AssertPropsMap((DataMap) _listener.AssertOneGetNewAndReset().Get("val0"), fieldsInner, new Object[] {"xE1x", -3});
        }
    
        [Test]
        public void TestDefaultColumnsAndSODA()
        {
            String epl = "select " +
                    "case TheString" +
                    " when \"A\" then new{TheString=\"Q\",IntPrimitive,col2=TheString||\"A\"}" +
                    " when \"B\" then new{TheString,IntPrimitive=10,col2=TheString||\"B\"} " +
                    "end as val0 from SupportBean as sb";
    
            EPStatement stmt = _epService.EPAdministrator.CreateEPL(epl);
            stmt.Events += _listener.Update;
            RunAssertionDefault(stmt);
    
            EPStatementObjectModel model = _epService.EPAdministrator.CompileEPL(epl);
            Assert.AreEqual(epl, model.ToEPL());
            stmt = _epService.EPAdministrator.Create(model);
            stmt.Events += _listener.Update;
            RunAssertionDefault(stmt);
    
            // test to-expression string
            epl = "select " +
                    "case TheString" +
                    " when \"A\" then new {TheString=\"Q\",IntPrimitive,col2=TheString||\"A\"}" +
                    " when \"B\" then new {TheString,IntPrimitive=10,col2=TheString||\"B\"} " +
                    "end from SupportBean as sb";
    
            stmt = _epService.EPAdministrator.CreateEPL(epl);
            stmt.Events += _listener.Update;
            Assert.AreEqual("case TheString when \"A\" then new{TheString=\"Q\",IntPrimitive,col2=TheString||\"A\"} when \"B\" then new{TheString,IntPrimitive=10,col2=TheString||\"B\"} end", stmt.EventType.PropertyNames[0]);
        }
    
        private void RunAssertionDefault(EPStatement stmt)
        {
            Assert.AreEqual(typeof(DataMap), stmt.EventType.GetPropertyType("val0"));
            FragmentEventType fragType = stmt.EventType.GetFragmentType("val0");
            Assert.IsFalse(fragType.IsIndexed);
            Assert.IsFalse(fragType.IsNative);
            Assert.AreEqual(typeof(string), fragType.FragmentType.GetPropertyType("TheString"));
            Assert.AreEqual(typeof(int?), fragType.FragmentType.GetPropertyType("IntPrimitive"));
            Assert.AreEqual(typeof(string), fragType.FragmentType.GetPropertyType("col2"));

            String[] fieldsInner = "TheString,IntPrimitive,col2".Split(',');
            _epService.EPRuntime.SendEvent(new SupportBean("E1", 1));
            ArrayAssertionUtil.AssertPropsMap((DataMap)_listener.AssertOneGetNewAndReset().Get("val0"), fieldsInner, new Object[] { null, null, null });
    
            _epService.EPRuntime.SendEvent(new SupportBean("A", 2));
            ArrayAssertionUtil.AssertPropsMap((DataMap)_listener.AssertOneGetNewAndReset().Get("val0"), fieldsInner, new Object[] { "Q", 2, "AA" });
    
            _epService.EPRuntime.SendEvent(new SupportBean("B", 3));
            ArrayAssertionUtil.AssertPropsMap((DataMap)_listener.AssertOneGetNewAndReset().Get("val0"), fieldsInner, new Object[] { "B", 10, "BB" });
    
            stmt.Dispose();
        }
    
        [Test]
        public void TestNewWithCase()
        {
            String epl = "select " +
                    "case " +
                    "  when TheString = 'A' then new { col1 = 'X', col2 = 10 } " +
                    "  when TheString = 'B' then new { col1 = 'Y', col2 = 20 } " +
                    "  when TheString = 'C' then new { col1 = null, col2 = null } " +
                    "  else new { col1 = 'Z', col2 = 30 } " +
                    "end as val0 from SupportBean sb";
            RunAssertion(epl);
    
            epl = "select " +
                    "case TheString " +
                    "  when 'A' then new { col1 = 'X', col2 = 10 } " +
                    "  when 'B' then new { col1 = 'Y', col2 = 20 } " +
                    "  when 'C' then new { col1 = null, col2 = null } " +
                    "  else new { col1 = 'Z', col2 = 30 } " +
                    "end as val0 from SupportBean sb";
            RunAssertion(epl);
        }
    
        private void RunAssertion(String epl)
        {
            EPStatement stmt = _epService.EPAdministrator.CreateEPL(epl);
            stmt.Events += _listener.Update;
            Assert.AreEqual(typeof(DataMap), stmt.EventType.GetPropertyType("val0"));
            FragmentEventType fragType = stmt.EventType.GetFragmentType("val0");
            Assert.IsFalse(fragType.IsIndexed);
            Assert.IsFalse(fragType.IsNative);
            Assert.AreEqual(typeof(string), fragType.FragmentType.GetPropertyType("col1"));
            Assert.AreEqual(typeof(int?), fragType.FragmentType.GetPropertyType("col2"));
    
            String[] fieldsInner = "col1,col2".Split(',');
            _epService.EPRuntime.SendEvent(new SupportBean("E1", 1));
            ArrayAssertionUtil.AssertPropsMap((DataMap)_listener.AssertOneGetNewAndReset().Get("val0"), fieldsInner, new Object[] { "Z", 30 });
    
            _epService.EPRuntime.SendEvent(new SupportBean("A", 2));
            ArrayAssertionUtil.AssertPropsMap((DataMap)_listener.AssertOneGetNewAndReset().Get("val0"), fieldsInner, new Object[] { "X", 10 });
    
            _epService.EPRuntime.SendEvent(new SupportBean("B", 3));
            ArrayAssertionUtil.AssertPropsMap((DataMap)_listener.AssertOneGetNewAndReset().Get("val0"), fieldsInner, new Object[] { "Y", 20 });
    
            _epService.EPRuntime.SendEvent(new SupportBean("C", 4));
            ArrayAssertionUtil.AssertPropsMap((DataMap)_listener.AssertOneGetNewAndReset().Get("val0"), fieldsInner, new Object[] { null, null });
    
            stmt.Dispose();
        }
    
        [Test]
        public void TestInvalid()
        {
            String epl;
    
            epl = "select case when true then new { col1 = 'a' } else 1 end from SupportBean";
            TryInvalid(epl, "Error starting statement: Failed to validate select-clause expression 'case when true then new{col1=\"a\"} e...(44 chars)': Case node 'when' expressions require that all results either return a single value or a Map-type (new-operator) value, check the else-condition [select case when true then new { col1 = 'a' } else 1 end from SupportBean]");
    
            epl = "select case when true then new { col1 = 'a' } when false then 1 end from SupportBean";
            TryInvalid(epl, "Error starting statement: Failed to validate select-clause expression 'case when true then new{col1=\"a\"} w...(55 chars)': Case node 'when' expressions require that all results either return a single value or a Map-type (new-operator) value, check when-condition number 1 [select case when true then new { col1 = 'a' } when false then 1 end from SupportBean]");
    
            epl = "select case when true then new { col1 = 'a' } else new { col1 = 1 } end from SupportBean";
            TryInvalid(epl, "Error starting statement: Failed to validate select-clause expression 'case when true then new{col1=\"a\"} e...(54 chars)': Incompatible case-when return types by new-operator in case-when number 1: Type by name 'Case-when number 1' in property 'col1' expected System.String but receives " + typeof(int?) + " [select case when true then new { col1 = 'a' } else new { col1 = 1 } end from SupportBean]");
    
            epl = "select case when true then new { col1 = 'a' } else new { col2 = 'a' } end from SupportBean";
            TryInvalid(epl, "Error starting statement: Failed to validate select-clause expression 'case when true then new{col1=\"a\"} e...(56 chars)': Incompatible case-when return types by new-operator in case-when number 1: The property 'col1' is not provided but required [select case when true then new { col1 = 'a' } else new { col2 = 'a' } end from SupportBean]");
    
            epl = "select case when true then new { col1 = 'a', col1 = 'b' } end from SupportBean";
            TryInvalid(epl, "Error starting statement: Failed to validate select-clause expression 'case when true then new{col1=\"a\",co...(46 chars)': Failed to validate new-keyword property names, property 'col1' has already been declared [select case when true then new { col1 = 'a', col1 = 'b' } end from SupportBean]");
        }
    
        private void TryInvalid(String epl, String message) {
            try
            {
                _epService.EPAdministrator.CreateEPL(epl);
                Assert.Fail();
            }
            catch (EPStatementException ex) {
                Assert.AreEqual(message, ex.Message);
            }
        }
    }
}
