///////////////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2006-2012 Esper Team. All rights reserved.                           /
// http://esper.codehaus.org                                                          /
// ---------------------------------------------------------------------------------- /
// The software in this package is published under the terms of the GPL license       /
// a copy of which has been included with this distribution in the license.txt file.  /
///////////////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Configuration;

using com.espertech.esper.client;
using com.espertech.esper.compat.collections;
using com.espertech.esper.epl.db;
using com.espertech.esper.support.schedule;

namespace com.espertech.esper.support.epl
{
    using Configuration = esper.client.Configuration;
    
    public class SupportDatabaseService
	{
        protected internal const String ESPER_TEST_CONFIG = "regression/esper.test.readconfig.cfg.xml";

	    public static readonly ConfigurationDBRef DbConfigReference1;
        public static readonly ConfigurationDBRef DbConfigReference2;

	    public static readonly DbDriverFactoryConnection DbDriverFactory1;
        public static readonly DbDriverFactoryConnection DbDriverFactory2;

        static SupportDatabaseService()
        {
            var configuration = ConfigurationManager.GetSection("esper-configuration") as Configuration;

            var dbTable = configuration.DatabaseReferences;
            DbConfigReference1 = dbTable["db1"];
            DbConfigReference2 = dbTable["db2"];

            DbDriverFactory1 = DbConfigReference1.ConnectionFactoryDesc as DbDriverFactoryConnection;
            DbDriverFactory2 = DbConfigReference2.ConnectionFactoryDesc as DbDriverFactoryConnection;
        }

        /// <summary>
        /// Gets the first database driver.
        /// </summary>
	    public static DbDriver Driver1
	    {
            get { return DbDriverFactory1.Driver; }
	    }

        /// <summary>
        /// Gets the second database driver.
        /// </summary>
        public static DbDriver Driver2
        {
            get { return DbDriverFactory2.Driver; }
        }

        public const String DBNAME_FULL = "mydb";
        public const String DBNAME_PART = "mydb2";

	    public static DatabaseConfigServiceImpl MakeService()
		{
            IDictionary<string, ConfigurationDBRef> configs = new Dictionary<String, ConfigurationDBRef>();

			ConfigurationDBRef config;

	        config = DbConfigReference1;
		    configs.Put(DBNAME_FULL, config);

            config = DbConfigReference2;
            configs.Put(DBNAME_PART, config);

			return new DatabaseConfigServiceImpl( configs, new SupportSchedulingServiceImpl(), null );
		}
	}
}
