///////////////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2006-2013 Esper Team. All rights reserved.                           /
// http://esper.codehaus.org                                                          /
// ---------------------------------------------------------------------------------- /
// The software in this package is published under the terms of the GPL license       /
// a copy of which has been included with this distribution in the license.txt file.  /
///////////////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;

using com.espertech.esper.client;
using com.espertech.esper.client.soda;
using com.espertech.esper.compat.collections;
using com.espertech.esper.epl.spec;

namespace com.espertech.esper.core.service
{
    /// <summary>
    /// Prepared statement implementation that stores the statement object model and a list 
    /// of substitution parameters, to be mapped into an internal representation upon creation.
    /// </summary>
    [Serializable]
    public class EPPreparedStatementImpl : EPPreparedStatement, EPOnDemandPreparedQueryParameterized
    {
        private readonly EPStatementObjectModel _model;
        private readonly IDictionary<int, SubstitutionParameterExpression> _subParams;
        private readonly String _optionalEPL;

        /// <summary>Ctor. </summary>
        /// <param name="model">is the statement object model</param>
        /// <param name="subParams">is the substitution parameter list</param>
        /// <param name="optionalEPL">the EPL provided if any</param>
        public EPPreparedStatementImpl(EPStatementObjectModel model, IDictionary<int, SubstitutionParameterExpression> subParams, String optionalEPL)
        {
            _model = model;
            _subParams = subParams;
            _optionalEPL = optionalEPL;
        }
    
        public void SetObject(int parameterIndex, Object value)
        {
            if (parameterIndex < 1)
            {
                throw new ArgumentException("Substitution parameter index starts at 1");
            }
            if (_subParams.Count == 0)
            {
                throw new ArgumentException("Statement does not have substitution parameters indicated by the '?' character");
            }
            if (parameterIndex > _subParams.Count)
            {
                throw new ArgumentException("Invalid substitution parameter index of " + parameterIndex + " supplied, the maximum for this statement is " + _subParams.Count);
            }
            SubstitutionParameterExpression subs = _subParams.Get(parameterIndex);
            subs.Constant = value;
        }

        /// <summary>Returns the statement object model for the prepared statement </summary>
        /// <value>object model</value>
        public EPStatementObjectModel Model
        {
            get { return _model; }
        }

        public string OptionalEPL
        {
            get { return _optionalEPL; }
        }
    }
}
