///////////////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2006-2014 Esper Team. All rights reserved.                           /
// http://esper.codehaus.org                                                          /
// ---------------------------------------------------------------------------------- /
// The software in this package is published under the terms of the GPL license       /
// a copy of which has been included with this distribution in the license.txt file.  /
///////////////////////////////////////////////////////////////////////////////////////

using System;

using com.espertech.esper.compat;
using com.espertech.esper.compat.collections;
using com.espertech.esper.core.context.util;
using com.espertech.esper.epl.agg.access;
using com.espertech.esper.epl.core;
using com.espertech.esper.epl.expression.core;
using com.espertech.esper.epl.table.mgmt;

namespace com.espertech.esper.epl.agg.service
{
    /// <summary>
    /// Implementation for handling aggregation with grouping by group-keys.
    /// </summary>
    public class AggSvcGroupByWTableFactory : AggregationServiceFactory
    {
        private readonly TableMetadata tableMetadata;
        private readonly TableColumnMethodPair[] methodPairs;
        private readonly AggregationAccessorSlotPair[] accessors;
        private readonly bool isJoin;
        private readonly int[] targetStates;
        private readonly ExprNode[] accessStateExpr;
        private readonly AggregationAgent[] agents;
    
        public AggSvcGroupByWTableFactory(TableMetadata tableMetadata, TableColumnMethodPair[] methodPairs, AggregationAccessorSlotPair[] accessors, bool join, int[] targetStates, ExprNode[] accessStateExpr, AggregationAgent[] agents) {
            this.tableMetadata = tableMetadata;
            this.methodPairs = methodPairs;
            this.accessors = accessors;
            isJoin = join;
            this.targetStates = targetStates;
            this.accessStateExpr = accessStateExpr;
            this.agents = agents;
        }
    
        public AggregationService MakeService(AgentInstanceContext agentInstanceContext, MethodResolutionService methodResolutionService) {
            TableStateInstanceGroupBy tableState = (TableStateInstanceGroupBy) agentInstanceContext.StatementContext.TableService.GetState(tableMetadata.TableName, agentInstanceContext.AgentInstanceId);
            return new AggSvcGroupByWTableImpl(tableMetadata, methodPairs, accessors, isJoin,
                    tableState, targetStates, accessStateExpr, agents);
        }
    }
}
