///////////////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2006-2013 Esper Team. All rights reserved.                           /
// http://esper.codehaus.org                                                          /
// ---------------------------------------------------------------------------------- /
// The software in this package is published under the terms of the GPL license       /
// a copy of which has been included with this distribution in the license.txt file.  /
///////////////////////////////////////////////////////////////////////////////////////

using System.Collections.Generic;
using System.Linq;

using com.espertech.esper.compat.collections;
using com.espertech.esper.epl.agg.access;
using com.espertech.esper.epl.expression;
using com.espertech.esper.epl.expression.baseagg;

namespace com.espertech.esper.epl.agg.service
{
    public class AggregationMultiFunctionAnalysisHelper
    {
        // handle accessor aggregation (direct data window by-group access to properties)
        public static AggregationMultiFunctionAnalysisResult AnalyzeAccessAggregations(IList<AggregationServiceAggExpressionDesc> aggregations)
        {
            var currentSlot = 0;
            var accessProviderSlots = new LinkedHashMap<AggregationStateKey, int?>();
            var accessorPairs = new List<AggregationAccessorSlotPair>();
            var stateFactories = new List<AggregationStateFactory>();
    
            foreach (var aggregation in aggregations)
            {
                var aggregateNode = aggregation.AggregationNode;
                if (!aggregateNode.Factory.IsAccessAggregation) {
                    continue;
                }
    
                var providerKey = aggregateNode.Factory.GetAggregationStateKey(false);
                var accessor = aggregateNode.Factory.Accessor;

                var slot = accessProviderSlots.Get(providerKey);
                if (slot == null) {
                    accessProviderSlots.Put(providerKey, currentSlot);
                    slot = currentSlot++;
                    var providerFactory = aggregateNode.Factory.GetAggregationStateFactory(false);
                    stateFactories.Add(providerFactory);
                }
    
                accessorPairs.Add(new AggregationAccessorSlotPair(slot.Value, accessor));
            }

            var pairs = accessorPairs.ToArray();
            var accessAggregations = stateFactories.ToArray();
            return new AggregationMultiFunctionAnalysisResult(pairs, accessAggregations);
        }
    }
}
