///////////////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2006-2013 Esper Team. All rights reserved.                           /
// http://esper.codehaus.org                                                          /
// ---------------------------------------------------------------------------------- /
// The software in this package is published under the terms of the GPL license       /
// a copy of which has been included with this distribution in the license.txt file.  /
///////////////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Reflection;

using XLR8.CGLib;

using com.espertech.esper.client;
using com.espertech.esper.compat.logging;
using com.espertech.esper.epl.db;
using com.espertech.esper.events;

namespace com.espertech.esper.epl.core
{
    /// <summary>
    /// Viewable providing historical data from a database.
    /// </summary>
    public abstract class MethodPollingExecStrategyBase : PollExecStrategy
    {
        private static readonly ILog Log = LogManager.GetLogger(MethodBase.GetCurrentMethod().DeclaringType);
    
        protected readonly EventAdapterService EventAdapterService;
        protected readonly FastMethod Method;
        protected readonly EventType EventType;

        protected MethodPollingExecStrategyBase(EventAdapterService eventAdapterService, FastMethod method, EventType eventType)
        {
            EventAdapterService = eventAdapterService;
            Method = method;
            EventType = eventType;
        }
    
        protected abstract IList<EventBean> HandleResult(Object invocationResult);
    
        protected bool CheckNonNullArrayValue(Object value) {
            if (value == null) {
                Log.Warn("Expected non-null return result from method '" + Method.Name + "', but received null array element value");
                return false;
            }
            return true;
        }
    
        public void Start()
        {
        }
    
        public void Done()
        {
        }
    
        public void Dispose()
        {
        }
    
        public IList<EventBean> Poll(Object[] lookupValues)
        {
            List<EventBean> rowResult = null;
            try {
                var invocationResult = Method.Invoke(null, lookupValues);
                if (invocationResult != null) {
                    return HandleResult(invocationResult);
                }
            }
            catch (TargetInvocationException ex) {
                throw new EPException("Method '" + Method.Name + "' of class '" + Method.Target.DeclaringType.FullName +
                        "' reported an exception: " + ex.InnerException, ex.InnerException);
            }
    
            return rowResult;
        }
    }
}
