///////////////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2006-2013 Esper Team. All rights reserved.                           /
// http://esper.codehaus.org                                                          /
// ---------------------------------------------------------------------------------- /
// The software in this package is published under the terms of the GPL license       /
// a copy of which has been included with this distribution in the license.txt file.  /
///////////////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Reflection;

using XLR8.CGLib;

using com.espertech.esper.client;
using com.espertech.esper.compat;
using com.espertech.esper.compat.logging;
using com.espertech.esper.compat.magic;
using com.espertech.esper.events;

namespace com.espertech.esper.epl.core
{
    using DataMap = IDictionary<string, object>;

    public class MethodPollingExecStrategyMapArray : MethodPollingExecStrategyBaseArray
    {
        private static readonly ILog Log = LogManager.GetLogger(MethodBase.GetCurrentMethod().DeclaringType);

        public MethodPollingExecStrategyMapArray(EventAdapterService eventAdapterService, FastMethod method, EventType eventType)
            : base(eventAdapterService, method, eventType)
        {
        }
    
        protected override EventBean GetEventBean(Object value)
        {
            var valueDataMap = value as DataMap;
            if (valueDataMap == null)
            {
                var valueType = value.GetType();
                if (valueType.IsGenericStringDictionary())
                {
                    valueDataMap = MagicMarker.GetStringDictionaryFactory(valueType).Invoke(value);
                }
            }

            if (valueDataMap == null)
            {
                Log.Warn("Expected map-type for value, but received type '" + value.GetType() + "'");
                throw new EPException("Expected map-type for value, but received type '" + value.GetType() + "'");
            }

            return EventAdapterService.AdapterForTypedMap(valueDataMap, EventType);
        }
    }
}
