///////////////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2006-2012 Esper Team. All rights reserved.                           /
// http://esper.codehaus.org                                                          /
// ---------------------------------------------------------------------------------- /
// The software in this package is published under the terms of the GPL license       /
// a copy of which has been included with this distribution in the license.txt file.  /
///////////////////////////////////////////////////////////////////////////////////////

namespace com.espertech.esper.epl.datetime.eval
{
    public class TimePeriod
    {
        public TimePeriod(int? years,
                          int? months,
                          int? weeks,
                          int? days,
                          int? hours,
                          int? minutes,
                          int? seconds,
                          int? milliseconds)
        {
            Years = years;
            Months = months;
            Weeks = weeks;
            Days = days;
            Hours = hours;
            Minutes = minutes;
            Seconds = seconds;
            Milliseconds = milliseconds;
        }

        public int? Years { get; private set; }

        public int? Months { get; private set; }

        public int? Weeks { get; private set; }

        public int? Days { get; private set; }

        public int? Hours { get; private set; }

        public int? Minutes { get; private set; }

        public int? Seconds { get; private set; }

        public int? Milliseconds { get; private set; }

        protected bool Equals(TimePeriod other)
        {
            return Years == other.Years && 
                Months == other.Months && 
                Weeks == other.Weeks && 
                Days == other.Days &&
                Hours == other.Hours && 
                Minutes == other.Minutes && 
                Seconds == other.Seconds &&
                Milliseconds == other.Milliseconds;
        }

        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj))
                return false;
            if (ReferenceEquals(this, obj))
                return true;
            if (obj.GetType() != this.GetType())
                return false;
            return Equals((TimePeriod) obj);
        }

        public override int GetHashCode()
        {
            unchecked
            {
                var hashCode = Years.GetHashCode();
                hashCode = (hashCode*397) ^ Months.GetHashCode();
                hashCode = (hashCode*397) ^ Weeks.GetHashCode();
                hashCode = (hashCode*397) ^ Days.GetHashCode();
                hashCode = (hashCode*397) ^ Hours.GetHashCode();
                hashCode = (hashCode*397) ^ Minutes.GetHashCode();
                hashCode = (hashCode*397) ^ Seconds.GetHashCode();
                hashCode = (hashCode*397) ^ Milliseconds.GetHashCode();
                return hashCode;
            }
        }

        public bool HasNegativeValues
        {
            get
            {
                if (Years != null && Years < 0)
                {
                    return true;
                }
                if (Months != null && Months < 0)
                {
                    return true;
                }
                if (Weeks != null && Weeks < 0)
                {
                    return true;
                }
                if (Days != null && Days < 0)
                {
                    return true;
                }
                if (Hours != null && Hours < 0)
                {
                    return true;
                }
                if (Minutes != null && Minutes < 0)
                {
                    return true;
                }
                if (Seconds != null && Seconds < 0)
                {
                    return true;
                }
                if (Milliseconds != null && Milliseconds < 0)
                {
                    return true;
                }
                return false;
            }
        }
    }
}