///////////////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2006-2013 Esper Team. All rights reserved.                           /
// http://esper.codehaus.org                                                          /
// ---------------------------------------------------------------------------------- /
// The software in this package is published under the terms of the GPL license       /
// a copy of which has been included with this distribution in the license.txt file.  /
///////////////////////////////////////////////////////////////////////////////////////

using System;

using com.espertech.esper.epl.agg.service;
using com.espertech.esper.epl.expression.baseagg;
using com.espertech.esper.epl.expression.core;

namespace com.espertech.esper.epl.expression.methodagg
{
    /// <summary>
    /// Represents the Avedev(...) aggregate function is an expression tree.
    /// </summary>
    [Serializable]
    public class ExprAvedevNode : ExprAggregateNodeBase
    {
        private readonly bool _hasFilter;

        /// <summary>
        /// Ctor.
        /// </summary>
        /// <param name="distinct">flag indicating unique or non-unique value aggregation</param>
        /// <param name="hasFilter">if set to <c>true</c> [has filter].</param>
        public ExprAvedevNode(bool distinct, bool hasFilter)
            : base(distinct)
        {
            _hasFilter = hasFilter;
        }
    
        public override AggregationMethodFactory ValidateAggregationChild(ExprValidationContext validationContext)
        {
            var childType = ValidateNumericChildAllowFilter(validationContext.StreamTypeService, _hasFilter);
            return new ExprAvedevNodeFactory(this, childType);
        }

        protected override bool EqualsNodeAggregateMethodOnly(ExprAggregateNode node)
        {
            return node is ExprAvedevNode;
        }

        public override string AggregationFunctionName
        {
            get { return "avedev"; }
        }

        public bool HasFilter
        {
            get { return _hasFilter; }
        }
    }
}
