///////////////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2006-2014 Esper Team. All rights reserved.                           /
// http://esper.codehaus.org                                                          /
// ---------------------------------------------------------------------------------- /
// The software in this package is published under the terms of the GPL license       /
// a copy of which has been included with this distribution in the license.txt file.  /
///////////////////////////////////////////////////////////////////////////////////////

using System;

using com.espertech.esper.epl.agg.service;
using com.espertech.esper.epl.expression.baseagg;
using com.espertech.esper.epl.expression.core;

namespace com.espertech.esper.epl.expression.methodagg
{
    /// <summary>
    /// Represents the "countever" aggregate function is an expression tree.
    /// </summary>
    [Serializable]
    public class ExprCountEverNode : ExprAggregateNodeBase
    {
        /// <summary>
        /// Ctor.
        /// </summary>
        /// <param name="distinct">flag indicating unique or non-unique value aggregation</param>
        public ExprCountEverNode(bool distinct)
            : base(distinct)
        {
        }
    
        public override AggregationMethodFactory ValidateAggregationChild(ExprValidationContext validationContext) 
        {
            if (ChildNodes.Length > 2) {
                throw new ExprValidationException("Count-ever aggregation node must have less then 2 child nodes");
            }
            if (base.isDistinct) {
                throw new ExprValidationException("Aggregation function '" + AggregationFunctionName + "' does now allow distinct");
            }
    
            bool ignoreNulls = false;
            if (ChildNodes.Length == 0) {
                // no parameters is allowed
            }
            else {
                ignoreNulls = !(ChildNodes[0] is ExprWildcard);
                if (ChildNodes.Length == 2) {
                    base.ValidateFilter(ChildNodes[1].ExprEvaluator);
                }
            }
            return new ExprCountEverNodeFactory(this, ignoreNulls);
        }

        public bool HasFilter
        {
            get { return ChildNodes.Length == 2; }
        }

        public override string AggregationFunctionName
        {
            get { return "countever"; }
        }

        protected override bool EqualsNodeAggregateMethodOnly(ExprAggregateNode node)
        {
            return node is ExprCountEverNode;
        }
    }
}
