///////////////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2006-2013 Esper Team. All rights reserved.                           /
// http://esper.codehaus.org                                                          /
// ---------------------------------------------------------------------------------- /
// The software in this package is published under the terms of the GPL license       /
// a copy of which has been included with this distribution in the license.txt file.  /
///////////////////////////////////////////////////////////////////////////////////////

using System;

using com.espertech.esper.epl.agg.service;
using com.espertech.esper.epl.expression.baseagg;
using com.espertech.esper.epl.expression.core;

namespace com.espertech.esper.epl.expression.methodagg
{
    /// <summary>
    /// Represents the Count(...) and Count(*) and Count(distinct ...) aggregate function is an expression tree.
    /// </summary>
    [Serializable]
    public class ExprCountNode : ExprAggregateNodeBase
    {
        private readonly bool _hasFilter;

        /// <summary>
        /// Ctor.
        /// </summary>
        /// <param name="distinct">flag indicating unique or non-unique value aggregation</param>
        /// <param name="hasFilter">if set to <c>true</c> [has filter].</param>
        public ExprCountNode(bool distinct, bool hasFilter) 
            : base(distinct)
        {
            _hasFilter = hasFilter;
        }
    
        public override AggregationMethodFactory ValidateAggregationChild(ExprValidationContext validationContext)
        {
            if (ChildNodes.Length > 2)
            {
                throw new ExprValidationException("Count node must have less then 2 child nodes");
            }
    
            Type childType = null;
            bool ignoreNulls = false;
    
            if (ChildNodes.Length == 0) {
                // defaults
            }
            else if (ChildNodes.Length == 1) {
                if (!_hasFilter) {
                    childType = ChildNodes[0].ExprEvaluator.ReturnType;
                    ignoreNulls = true;
                }
                else {
                    base.ValidateFilter(ChildNodes[0].ExprEvaluator);
                }
            }
            else if (ChildNodes.Length == 2) {
                childType = ChildNodes[0].ExprEvaluator.ReturnType;
                ignoreNulls = true;
            }
            return new ExprCountNodeFactory(this, ignoreNulls, childType);
        }

        public override string AggregationFunctionName
        {
            get { return "count"; }
        }

        public bool HasFilter
        {
            get { return _hasFilter; }
        }

        protected override bool EqualsNodeAggregateMethodOnly(ExprAggregateNode node)
        {
            return node is ExprCountNode;
        }
    }
}
