///////////////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2006-2013 Esper Team. All rights reserved.                           /
// http://esper.codehaus.org                                                          /
// ---------------------------------------------------------------------------------- /
// The software in this package is published under the terms of the GPL license       /
// a copy of which has been included with this distribution in the license.txt file.  /
///////////////////////////////////////////////////////////////////////////////////////

using System;

using com.espertech.esper.epl.agg.service;
using com.espertech.esper.epl.expression.baseagg;
using com.espertech.esper.epl.expression.core;

namespace com.espertech.esper.epl.expression.methodagg
{
    /// <summary>Represents the "firstever" aggregate function is an expression tree. </summary>
    [Serializable]
    public class ExprFirstEverNode : ExprAggregateNodeBase
    {
        /// <summary>Ctor. </summary>
        /// <param name="distinct">flag indicating unique or non-unique value aggregation</param>
        public ExprFirstEverNode(bool distinct)
            : base(distinct)
        {
        }
    
        public override AggregationMethodFactory ValidateAggregationChild(ExprValidationContext validationContext)
        {
            if (ChildNodes.Length > 2)
            {
                throw new ExprValidationException("First aggregation node must have less then 2 child nodes");
            }
            if (ChildNodes.Length == 2)
            {
                base.ValidateFilter(ChildNodes[1].ExprEvaluator);
            }
            return new ExprFirstEverNodeFactory(this, ChildNodes[0].ExprEvaluator.ReturnType);
        }

        public bool HasFilter
        {
            get { return ChildNodes.Length == 2; }
        }

        public override string AggregationFunctionName
        {
            get { return "firstever"; }
        }

        protected override bool EqualsNodeAggregateMethodOnly(ExprAggregateNode node)
        {
            return node is ExprFirstEverNode;
        }
    }
}
