///////////////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2006-2014 Esper Team. All rights reserved.                           /
// http://esper.codehaus.org                                                          /
// ---------------------------------------------------------------------------------- /
// The software in this package is published under the terms of the GPL license       /
// a copy of which has been included with this distribution in the license.txt file.  /
///////////////////////////////////////////////////////////////////////////////////////

using System;

using com.espertech.esper.compat;
using com.espertech.esper.compat.collections;
using com.espertech.esper.epl.agg.service;
using com.espertech.esper.epl.expression.baseagg;
using com.espertech.esper.epl.expression.core;

namespace com.espertech.esper.epl.expression.methodagg
{
	/// <summary>
	/// Represents the "lastever" aggregate function is an expression tree.
	/// </summary>
    [Serializable]
    public class ExprLastEverNode : ExprAggregateNodeBase
	{
	    /// <summary>
	    /// Ctor.
	    /// </summary>
	    /// <param name="distinct">flag indicating unique or non-unique value aggregation</param>
	    public ExprLastEverNode(bool distinct)
            : base(distinct)
	    {
	    }

	    public override AggregationMethodFactory ValidateAggregationChild(ExprValidationContext validationContext)
	    {
	        if (this.ChildNodes.Length == 0 || this.ChildNodes.Length > 2)
	        {
	            throw new ExprValidationException("Last aggregation function must one or two parameters");
	        }
	        if (this.ChildNodes.Length == 2) {
	            base.ValidateFilter(this.ChildNodes[1].ExprEvaluator);
	        }
	        return new ExprLastEverNodeFactory(this, this.ChildNodes[0].ExprEvaluator.ReturnType);
	    }

	    public bool HasFilter
	    {
	        get { return this.ChildNodes.Length == 2; }
	    }

	    public override string AggregationFunctionName
	    {
	        get { return "lastever"; }
	    }

	    protected override bool EqualsNodeAggregateMethodOnly(ExprAggregateNode node)
	    {
	        return node is ExprLastEverNode;
	    }
	}
} // end of namespace
