///////////////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2006-2013 Esper Team. All rights reserved.                           /
// http://esper.codehaus.org                                                          /
// ---------------------------------------------------------------------------------- /
// The software in this package is published under the terms of the GPL license       /
// a copy of which has been included with this distribution in the license.txt file.  /
///////////////////////////////////////////////////////////////////////////////////////

using System;

using com.espertech.esper.epl.agg.service;
using com.espertech.esper.epl.expression.baseagg;
using com.espertech.esper.epl.expression.core;

namespace com.espertech.esper.epl.expression.methodagg
{
    /// <summary>
    /// Represents the Median(...) aggregate function is an expression tree.
    /// </summary>
    [Serializable]
    public class ExprMedianNode : ExprAggregateNodeBase
    {
        private readonly bool _hasFilter;

        /// <summary>
        /// Ctor.
        /// </summary>
        /// <param name="distinct">flag indicating unique or non-unique value aggregation</param>
        /// <param name="hasFilter">if set to <c>true</c> [has filter].</param>
        public ExprMedianNode(bool distinct, bool hasFilter)
            : base(distinct)
        {
            _hasFilter = hasFilter;
        }
    
        public override AggregationMethodFactory ValidateAggregationChild(ExprValidationContext validationContext)
        {
            Type childType = base.ValidateNumericChildAllowFilter(validationContext.StreamTypeService, _hasFilter);
            return new ExprMedianNodeFactory(this, childType);
        }

        public override string AggregationFunctionName
        {
            get { return "median"; }
        }

        protected override bool EqualsNodeAggregateMethodOnly(ExprAggregateNode node)
        {
            if (!(node is ExprMedianNode))
            {
                return false;
            }
    
            return true;
        }

        public bool HasFilter
        {
            get { return _hasFilter; }
        }
    }
}
