///////////////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2006-2014 Esper Team. All rights reserved.                           /
// http://esper.codehaus.org                                                          /
// ---------------------------------------------------------------------------------- /
// The software in this package is published under the terms of the GPL license       /
// a copy of which has been included with this distribution in the license.txt file.  /
///////////////////////////////////////////////////////////////////////////////////////

using System;

using com.espertech.esper.compat;
using com.espertech.esper.compat.collections;
using com.espertech.esper.epl.agg.access;
using com.espertech.esper.epl.agg.aggregator;
using com.espertech.esper.epl.agg.service;
using com.espertech.esper.epl.core;
using com.espertech.esper.epl.expression.baseagg;
using com.espertech.esper.epl.expression.core;
using com.espertech.esper.type;

namespace com.espertech.esper.epl.expression.methodagg
{
    [Serializable]
    public class ExprMinMaxAggrNodeFactory : AggregationMethodFactory
	{
	    private readonly ExprMinMaxAggrNode parent;
	    private readonly Type type;
	    private readonly bool hasDataWindows;

	    public ExprMinMaxAggrNodeFactory(ExprMinMaxAggrNode parent, Type type, bool hasDataWindows) {
	        this.parent = parent;
	        this.type = type;
	        this.hasDataWindows = hasDataWindows;
	    }

        public bool IsAccessAggregation
        {
            get { return false; }
        }

        public AggregationStateKey GetAggregationStateKey(bool isMatchRecognize) {
	        throw new IllegalStateException("Not an access aggregation function");
	    }

	    public AggregationStateFactory GetAggregationStateFactory(bool isMatchRecognize) {
	        throw new IllegalStateException("Not an access aggregation function");
	    }

        public AggregationAccessor Accessor
        {
            get { throw new IllegalStateException("Not an access aggregation function"); }
        }

        public Type ResultType
        {
            get { return type; }
        }

        public AggregationMethod Make(MethodResolutionService methodResolutionService, int agentInstanceId, int groupId, int aggregationId) {
	        AggregationMethod method = methodResolutionService.MakeMinMaxAggregator(agentInstanceId, groupId, aggregationId, parent.MinMaxTypeEnum, type, hasDataWindows, parent.HasFilter);
	        if (!parent.IsDistinct) {
	            return method;
	        }
	        return methodResolutionService.MakeDistinctAggregator(agentInstanceId, groupId, aggregationId, method, type, parent.HasFilter);
	    }

        public ExprAggregateNodeBase AggregationExpression
        {
            get { return parent; }
        }

        public void ValidateIntoTableCompatible(AggregationMethodFactory intoTableAgg)
        {
	        AggregationMethodFactoryUtil.ValidateAggregationType(this, intoTableAgg);
	        ExprMinMaxAggrNodeFactory that = (ExprMinMaxAggrNodeFactory) intoTableAgg;
	        AggregationMethodFactoryUtil.ValidateAggregationInputType(type, that.type);
	        AggregationMethodFactoryUtil.ValidateAggregationFilter(parent.HasFilter, that.parent.HasFilter);
	        if (parent.MinMaxTypeEnum != that.parent.MinMaxTypeEnum) {
	            throw new ExprValidationException("The aggregation declares " +
	                    parent.MinMaxTypeEnum.GetExpressionText() +
	                    " and provided is " +
	                    that.parent.MinMaxTypeEnum.GetExpressionText());
	        }
	        AggregationMethodFactoryUtil.ValidateAggregationUnbound(hasDataWindows, that.hasDataWindows);
	    }

        public AggregationAgent AggregationStateAgent
        {
            get { return null; }
        }
	}
} // end of namespace
