///////////////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2006-2013 Esper Team. All rights reserved.                           /
// http://esper.codehaus.org                                                          /
// ---------------------------------------------------------------------------------- /
// The software in this package is published under the terms of the GPL license       /
// a copy of which has been included with this distribution in the license.txt file.  /
///////////////////////////////////////////////////////////////////////////////////////

using System;

using com.espertech.esper.compat;
using com.espertech.esper.epl.agg.service;
using com.espertech.esper.epl.expression.baseagg;
using com.espertech.esper.epl.expression.core;

namespace com.espertech.esper.epl.expression.methodagg
{
    /// <summary>
    /// Represents the Nth(...) and aggregate function is an expression tree.
    /// </summary>
    [Serializable]
    public class ExprNthAggNode : ExprAggregateNodeBase
    {
        /// <summary>Ctor. </summary>
        /// <param name="distinct">flag indicating unique or non-unique value aggregation</param>
        public ExprNthAggNode(bool distinct)
            : base(distinct)
        {
        }
    
        public override AggregationMethodFactory ValidateAggregationChild(ExprValidationContext validationContext)
        {
            const string message = "The nth aggregation function requires two parameters, an expression returning aggregation values and a numeric index constant";
            if (ChildNodes.Length != 2) {
                throw new ExprValidationException(message);
            }
    
            ExprNode first = ChildNodes[0];
            ExprNode second = ChildNodes[1];
            if (!second.IsConstantResult) {
                throw new ExprValidationException(message);
            }

            var num = second.ExprEvaluator.Evaluate(new EvaluateParams(null, true, validationContext.ExprEvaluatorContext));
            int size = num.AsInt();
    
            return new ExprNthAggNodeFactory(this, first.ExprEvaluator.ReturnType, size);
        }

        public override string AggregationFunctionName
        {
            get { return "nth"; }
        }

        protected override bool EqualsNodeAggregateMethodOnly(ExprAggregateNode node)
        {
            return node is ExprNthAggNode;
        }
    }
}
