///////////////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2006-2014 Esper Team. All rights reserved.                           /
// http://esper.codehaus.org                                                          /
// ---------------------------------------------------------------------------------- /
// The software in this package is published under the terms of the GPL license       /
// a copy of which has been included with this distribution in the license.txt file.  /
///////////////////////////////////////////////////////////////////////////////////////

using System;

using Antlr4.Runtime;

using com.espertech.esper.epl.generated;
using com.espertech.esper.epl.spec;

namespace com.espertech.esper.epl.parse
{
    /// <summary>
    /// Helper class for walking the match-recognize AST.
    /// </summary>
    public class ASTMatchRecognizeHelper
    {
        private const String Message = "Match-recognize AFTER clause must be either AFTER MATCH SKIP TO LAST ROW or AFTER MATCH SKIP TO NEXT ROW or AFTER MATCH SKIP TO CURRENT ROW";
    
        public static MatchRecognizeSkipEnum ParseSkip(CommonTokenStream tokenStream, EsperEPL2GrammarParser.MatchRecogMatchesAfterSkipContext ctx)
        {
            if ((!ctx.i1.GetText().ToUpper().Equals("MATCH")) ||
                (!ctx.i2.GetText().ToUpper().Equals("SKIP")) ||
                (!ctx.i5.GetText().ToUpper().Equals("ROW"))
                )
            {
                throw ASTWalkException.From(Message, tokenStream, ctx);
            }
    
            if ((!ctx.i3.GetText().ToUpper().Equals("TO")) &&
                (!ctx.i3.GetText().ToUpper().Equals("PAST"))
                )
            {
                throw ASTWalkException.From(Message, tokenStream, ctx);
            }
    
            if (ctx.i4.GetText().ToUpper().Equals("LAST"))
            {
                return MatchRecognizeSkipEnum.PAST_LAST_ROW;
            }
            else if (ctx.i4.GetText().ToUpper().Equals("NEXT"))
            {
                return MatchRecognizeSkipEnum.TO_NEXT_ROW;
            }
            else if (ctx.i4.GetText().ToUpper().Equals("CURRENT"))
            {
                return MatchRecognizeSkipEnum.TO_CURRENT_ROW;
            }
            throw ASTWalkException.From(Message);
        }
    }
}
