///////////////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2006-2012 Esper Team. All rights reserved.                           /
// http://esper.codehaus.org                                                          /
// ---------------------------------------------------------------------------------- /
// The software in this package is published under the terms of the GPL license       /
// a copy of which has been included with this distribution in the license.txt file.  /
///////////////////////////////////////////////////////////////////////////////////////

using com.espertech.esper.client;
using com.espertech.esper.epl.expression.core;
using com.espertech.esper.epl.expression;

namespace com.espertech.esper.epl.property
{
    /// <summary>
    /// Interface for a function that evaluates the properties of an event and returns event representing the properties.
    /// </summary>
    public interface PropertyEvaluator
    {
        /// <summary>
        /// Returns the result events based on property values, or null if none found.
        /// </summary>
        /// <param name="theEvent">to inspect</param>
        /// <param name="exprEvaluatorContext">expression evaluation context</param>
        /// <returns>events representing Property(s)</returns>
        EventBean[] GetProperty(EventBean theEvent, ExprEvaluatorContext exprEvaluatorContext);

        /// <summary>
        /// Returns the result type of the events generated by evaluating a property expression.
        /// </summary>
        /// <value>result event type</value>
        EventType FragmentEventType { get; }

        /// <summary>
        /// Compare to another property evaluator.
        /// </summary>
        /// <param name="otherFilterPropertyEval">other</param>
        /// <returns>equals or not</returns>
        bool CompareTo(PropertyEvaluator otherFilterPropertyEval);
    }
}
