///////////////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2006-2013 Esper Team. All rights reserved.                           /
// http://esper.codehaus.org                                                          /
// ---------------------------------------------------------------------------------- /
// The software in this package is published under the terms of the GPL license       /
// a copy of which has been included with this distribution in the license.txt file.  /
///////////////////////////////////////////////////////////////////////////////////////

using System;
using System.IO;

using com.espertech.esper.epl.expression;
using com.espertech.esper.epl.expression.core;
using com.espertech.esper.epl.spec;
using com.espertech.esper.pattern.guard;

namespace com.espertech.esper.pattern
{
    /// <summary>
    /// This class represents a guard in the evaluation tree representing an event expressions.
    /// </summary>
    [Serializable]
    public class EvalGuardFactoryNode : EvalNodeFactoryBase
    {
        private readonly PatternGuardSpec _patternGuardSpec;
        [NonSerialized] private GuardFactory _guardFactory;

        /// <summary>Constructor. </summary>
        /// <param name="patternGuardSpec">factory for guard construction</param>
        public EvalGuardFactoryNode(PatternGuardSpec patternGuardSpec)
        {
            _patternGuardSpec = patternGuardSpec;
        }

        public override EvalNode MakeEvalNode(PatternAgentInstanceContext agentInstanceContext)
        {
            EvalNode child = EvalNodeUtil.MakeEvalNodeSingleChild(this.ChildNodes, agentInstanceContext);
            return new EvalGuardNode(agentInstanceContext, this, child);
        }

        /// <summary>Returns the guard object specification to use for instantiating the guard factory and guard. </summary>
        /// <value>guard specification</value>
        public PatternGuardSpec PatternGuardSpec
        {
            get { return _patternGuardSpec; }
        }

        /// <summary>Supplies the guard factory to the node. </summary>
        /// <value>is the guard factory</value>
        public GuardFactory GuardFactory
        {
            set { _guardFactory = value; }
            get { return _guardFactory; }
        }

        public override String ToString()
        {
            return ("EvalGuardNode guardFactory=" + _guardFactory +
                    "  children=" + ChildNodes.Count);
        }

        public override bool IsFilterChildNonQuitting
        {
            get { return false; }
        }

        public override bool IsStateful
        {
            get { return true; }
        }

        public override void ToPrecedenceFreeEPL(TextWriter writer)
        {
            ChildNodes[0].ToEPL(writer, Precedence);
            if (_patternGuardSpec.ObjectNamespace.Equals(GuardEnum.WHILE_GUARD.Namespace) &&
                _patternGuardSpec.ObjectName.Equals(GuardEnum.WHILE_GUARD.Name))
            {
                writer.Write(" while ");
            }
            else
            {
                writer.Write(" where ");
                writer.Write(_patternGuardSpec.ObjectNamespace);
                writer.Write(":");
                writer.Write(_patternGuardSpec.ObjectName);
            }
            writer.Write("(");
            ExprNodeUtility.ToExpressionStringParameterList(_patternGuardSpec.ObjectParameters, writer);
            writer.Write(")");
        }

        public override PatternExpressionPrecedenceEnum Precedence
        {
            get { return PatternExpressionPrecedenceEnum.GUARD_POSTFIX; }
        }
    }
}