///////////////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2006-2012 Esper Team. All rights reserved.                           /
// http://esper.codehaus.org                                                          /
// ---------------------------------------------------------------------------------- /
// The software in this package is published under the terms of the GPL license       /
// a copy of which has been included with this distribution in the license.txt file.  /
///////////////////////////////////////////////////////////////////////////////////////


using System;
using System.Linq;

namespace com.espertech.esper.pattern.guard
{
    /// <summary> 
    /// Enum for all build-in guards.
    /// </summary>

    public class GuardEnum
    {
        /// <summary>
        /// Timer guard.
        /// </summary>

        public static readonly GuardEnum TIMER_WITHIN =
            new GuardEnum("timer", "within");
        public static readonly GuardEnum TIMER_WITHINMAX =
            new GuardEnum("timer", "withinmax");
        public static readonly GuardEnum WHILE_GUARD =
            new GuardEnum("internal", "while");

        /// <summary>
        /// All values available through this pseudo-enum
        /// </summary>
        
        public static readonly GuardEnum[] Values = new[]
        {
        	TIMER_WITHIN,
            TIMER_WITHINMAX,
            WHILE_GUARD
        };

        /// <summary>
        /// Initializes a new instance of the <see cref="GuardEnum"/> class.
        /// </summary>
        /// <param name="nspace">The nspace.</param>
        /// <param name="name">The name.</param>
        GuardEnum(String nspace, String name)
        {
            Namespace = nspace;
            Name = name;
        }

        /// <summary>
        /// Gets the namespace.
        /// </summary>
        /// <value>The namespace.</value>
        public string Namespace { get; private set; }

        /// <summary>
        /// Gets the name.
        /// </summary>
        /// <value>The name.</value>
        public string Name { get; private set; }

        /// <summary>Returns the enum for the given namespace and name.</summary>
        /// <param name="nspace">guard namespace</param>
        /// <param name="name">guard name</param>
        /// <returns>enum</returns>

        public static GuardEnum ForName(String nspace, String name)
        {
            return Values.FirstOrDefault(
                guardEnum => (guardEnum.Namespace == nspace) && (guardEnum.Name == name));
        }

        public static bool IsWhile(String nspace, String name)
        {
            return 
                (nspace == WHILE_GUARD.Namespace) &&
                (name == WHILE_GUARD.Name);
        }
    }

    public static class GuardEnumExtensions
    {
        /// <summary>
        /// Gets the class associated with the guard enum.
        /// </summary>
        /// <param name="guardEnum">The guard enum.</param>
        /// <returns></returns>
        public static Type GetClazz(this GuardEnum guardEnum)
        {
            if (ReferenceEquals(guardEnum, GuardEnum.TIMER_WITHIN))
                return typeof(TimerWithinGuardFactory);
            if (ReferenceEquals(guardEnum, GuardEnum.TIMER_WITHINMAX))
                return typeof(TimerWithinOrMaxCountGuardFactory);
            if (ReferenceEquals(guardEnum, GuardEnum.WHILE_GUARD))
                return typeof(ExpressionGuardFactory);

            throw new ArgumentException("invalid value", "guardEnum");
        }
    }
}
