///////////////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2006-2012 Esper Team. All rights reserved.                           /
// http://esper.codehaus.org                                                          /
// ---------------------------------------------------------------------------------- /
// The software in this package is published under the terms of the GPL license       /
// a copy of which has been included with this distribution in the license.txt file.  /
///////////////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;

using com.espertech.esper.compat.logging;
using com.espertech.esper.util;

namespace com.espertech.esper.rowregex
{
    /// <summary>
    /// Base node for
    /// </summary>
    [Serializable]
    public abstract class RowRegexExprNode : MetaDefItem
    {
        private static readonly ILog Log = LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        /// <summary>
        /// Returns the expression node rendered as a string.
        /// </summary>
        /// <returns>
        /// string rendering of expression
        /// </returns>
        public abstract String ToExpressionString();

        /// <summary>
        /// Constructor creates a list of child nodes.
        /// </summary>
        protected RowRegexExprNode()
        {
            ChildNodes = new List<RowRegexExprNode>();
        }

        /// <summary>
        /// Adds a child node.
        /// </summary>
        /// <param name="childNode">is the child evaluation tree node to add</param>
        public void AddChildNode(RowRegexExprNode childNode)
        {
            ChildNodes.Add(childNode);
        }

        /// <summary>
        /// Returns list of child nodes.
        /// </summary>
        /// <returns>
        /// list of child nodes
        /// </returns>
        public IList<RowRegexExprNode> ChildNodes { get; private set; }

        /// <summary>
        /// Recursively print out all nodes.
        /// </summary>
        /// <param name="prefix">is printed out for naming the printed info</param>
        public void DumpDebug(String prefix)
        {
            if (Log.IsDebugEnabled)
            {
                Log.Debug(".DumpDebug {0}{1}", prefix, this);
            }
            foreach (RowRegexExprNode node in ChildNodes)
            {
                node.DumpDebug(prefix + "  ");
            }
        }
    }
}
